@@ -4,7 +4,7 @@
  * Run it with an 'help', or an invalid option and it will tell you what options
  * it accepts.
  *
- * March 2002, Igor Zlatkovic <igor@stud.fh-frankfurt.de>
+ * March 2002, Igor Zlatkovic <igor@zlatkovic.com>
  */
 
 /* The source directory, relative to the one where this file resides. */
@@ -17,13 +17,10 @@ var baseName = "libxml2";
 /* Configure file which contains the version and the output file where
    we can store our build configuration. */
 var configFile = srcDirXml + "\\configure.in";
-var versionFile = ".\\configure.txt";
-/* Input and output files regarding the libxml features. The second
-   output file is there for the compatibility reasons, otherwise it
-   is identical to the first. */
+var versionFile = ".\\config.msvc";
+/* Input and output files regarding the libxml features. */
 var optsFileIn = srcDirXml + "\\include\\libxml\\xmlversion.h.in";
 var optsFile = srcDirXml + "\\include\\libxml\\xmlversion.h";
-var optsFile2 = srcDirXml + "\\include\\libxml\\xmlwin32version.h";
 /* Version strings for the binary distribution. Will be filled later 
    in the code. */
 var verMajor;
@@ -31,7 +28,7 @@ var verMinor;
 var verMicro;
 /* Libxml features. */
 var withTrio = false;
-var withThreads = false;
+var withThreads = "native";
 var withFtp = true;
 var withHttp = true;
 var withHtml = true;
@@ -47,7 +44,9 @@ var withDebug = true;
 var withMemDebug = false;
 var withSchemas = true;
 var withRegExps = true;
+var withPython = false;
 /* Win32 build options. */
+var compiler = "msvc";
 var buildDebug = 0;
 var buildStatic = 0;
 var buildPrefix = ".";
@@ -65,11 +64,11 @@ var error = 0;
 function boolToStr(opt)
 {
 	if (opt == false)
-		return "Disabled";
+		return "no";
 	else if (opt == true)
-		return "Enabled";
+		return "yes";
 	error = 1;
-	return "Undefined";
+	return "*** undefined ***";
 }
 
 /* Helper function, transforms the argument string into a boolean
@@ -92,10 +91,10 @@ function usage()
 	txt += "  cscript " + WScript.ScriptName + " <options>\n";
 	txt += "  cscript " + WScript.ScriptName + " help\n\n";
 	txt += "Options can be specified in the form <option>=<value>, where the value is\n";
-	txt += "either 'yes' or 'no'.\n\n";
-	txt += "XML processor options, default value given in parentheses:\n\n";
+	txt += "either 'yes' or 'no', if not stated otherwise.\n\n";
+	txt += "\nXML processor options, default value given in parentheses:\n\n";
 	txt += "  trio:       Enable TRIO string manipulator (" + (withTrio? "yes" : "no")  + ")\n";
-	txt += "  threads:    Enable thread safety (" + (withThreads? "yes" : "no")  + ") \n";
+	txt += "  threads:    Enable thread safety [no|ctls|native|posix] (" + (withThreads)  + ") \n";
 	txt += "  ftp:        Enable FTP client (" + (withFtp? "yes" : "no")  + ")\n";
 	txt += "  http:       Enable HTTP client (" + (withHttp? "yes" : "no")  + ")\n";
 	txt += "  html:       Enable HTML processor (" + (withHtml? "yes" : "no")  + ")\n";
@@ -111,7 +110,9 @@ function usage()
 	txt += "  mem_debug:  Enable memory debugger (" + (withMemDebug? "yes" : "no")  + ")\n";
 	txt += "  regexps:    Enable regular expressions (" + (withRegExps? "yes" : "no") + ")\n";
 	txt += "  schemas:    Enable XML Schema support (" + (withSchemas? "yes" : "no")  + ")\n";
+	txt += "  python:     Build Python bindings (" + (withPython? "yes" : "no")  + ")\n";
 	txt += "\nWin32 build options, default value given in parentheses:\n\n";
+	txt += "  compiler:   Compiler to be used [msvc|mingw|bcb] (" + compiler + ")\n";
 	txt += "  debug:      Build unoptimised debug executables (" + (buildDebug? "yes" : "no")  + ")\n";
 	txt += "  static:     Link xmllint statically to libxml2 (" + (buildStatic? "yes" : "no")  + ")\n";
 	txt += "  prefix:     Base directory for the installation (" + buildPrefix + ")\n";
@@ -138,6 +139,12 @@ function discoverVersion()
 	var fso, cf, vf, ln, s;
 	fso = new ActiveXObject("Scripting.FileSystemObject");
 	cf = fso.OpenTextFile(configFile, 1);
+	if (compiler == "msvc")
+		versionFile = ".\\config.msvc";
+	else if (compiler == "mingw")
+		versionFile = ".\\config.mingw";
+	else if (compiler == "bcb")
+		versionFile = ".\\config.bcb";
 	vf = fso.CreateTextFile(versionFile, true);
 	vf.WriteLine("# " + versionFile);
 	vf.WriteLine("# This file is generated automatically by " + WScript.ScriptName + ".");
@@ -161,7 +168,7 @@ function discoverVersion()
 	vf.WriteLine("UTILS_SRCDIR=" + srcDirUtils);
 	vf.WriteLine("BINDIR=" + binDir);
 	vf.WriteLine("WITH_TRIO=" + (withTrio? "1" : "0"));
-	vf.WriteLine("WITH_THREADS=" + (withThreads? "1" : "0"));
+	vf.WriteLine("WITH_THREADS=" + withThreads);
 	vf.WriteLine("WITH_FTP=" + (withFtp? "1" : "0"));
 	vf.WriteLine("WITH_HTTP=" + (withHttp? "1" : "0"));
 	vf.WriteLine("WITH_HTML=" + (withHtml? "1" : "0"));
@@ -177,6 +184,7 @@ function discoverVersion()
 	vf.WriteLine("WITH_MEM_DEBUG=" + (withMemDebug? "1" : "0"));
 	vf.WriteLine("WITH_SCHEMAS=" + (withSchemas? "1" : "0"));
 	vf.WriteLine("WITH_REGEXPS=" + (withRegExps? "1" : "0"));
+	vf.WriteLine("WITH_PYTHON=" + (withPython? "1" : "0"));
 	vf.WriteLine("DEBUG=" + (buildDebug? "1" : "0"));
 	vf.WriteLine("STATIC=" + (buildStatic? "1" : "0"));
 	vf.WriteLine("PREFIX=" + buildPrefix);
@@ -184,8 +192,17 @@ function discoverVersion()
 	vf.WriteLine("INCPREFIX=" + buildIncPrefix);
 	vf.WriteLine("LIBPREFIX=" + buildLibPrefix);
 	vf.WriteLine("SOPREFIX=" + buildSoPrefix);
-	vf.WriteLine("INCLUDE=$(INCLUDE);" + buildInclude);
-	vf.WriteLine("LIB=$(LIB);" + buildLib);
+	if (compiler == "msvc") {
+		vf.WriteLine("INCLUDE=$(INCLUDE);" + buildInclude);
+		vf.WriteLine("LIB=$(LIB);" + buildLib);
+	} else if (compiler == "mingw") {
+		vf.WriteLine("INCLUDE+=;" + buildInclude);
+		vf.WriteLine("LIB+=;" + buildLib);
+	}
+	} else if (compiler == "bcb") {
+		vf.WriteLine("INCLUDE=" + buildInclude);
+		vf.WriteLine("LIB=" + buildLib);
+	}
 	vf.Close();
 }
 
@@ -209,7 +226,7 @@ function configureLibxml()
 		} else if (s.search(/\@WITH_TRIO\@/) != -1) {
 			of.WriteLine(s.replace(/\@WITH_TRIO\@/, withTrio? "1" : "0"));
 		} else if (s.search(/\@WITH_THREADS\@/) != -1) {
-			of.WriteLine(s.replace(/\@WITH_THREADS\@/, withThreads? "1" : "0"));
+			of.WriteLine(s.replace(/\@WITH_THREADS\@/, withThreads == "no"? "0" : "1"));
 		} else if (s.search(/\@WITH_FTP\@/) != -1) {
 			of.WriteLine(s.replace(/\@WITH_FTP\@/, withFtp? "1" : "0"));
 		} else if (s.search(/\@WITH_HTTP\@/) != -1) {
@@ -245,7 +262,31 @@ function configureLibxml()
 	}
 	ofi.Close();
 	of.Close();
-	fso.CopyFile(optsFile, optsFile2, true);
+}
+/* Configures Python bindings. Otherwise identical to the above */
+function configureLibxmlPy()
+{
+	var pyOptsFileIn = srcDirXml + "\\python\\setup.py.in";
+	var pyOptsFile = srcDirXml + "\\python\\setup.py";
+	var fso, ofi, of, ln, s;
+	fso = new ActiveXObject("Scripting.FileSystemObject");
+	ofi = fso.OpenTextFile(pyOptsFileIn, 1);
+	of = fso.CreateTextFile(pyOptsFile, true);
+	while (ofi.AtEndOfStream != true) {
+		ln = ofi.ReadLine();
+		s = new String(ln);
+		if (s.search(/\@LIBXML_VERSION\@/) != -1) {
+			of.WriteLine(s.replace(/\@LIBXML_VERSION\@/, 
+				verMajor + "." + verMinor + "." + verMicro));
+		} else if (s.search(/\@prefix\@/) != -1) {
+			of.WriteLine(s.replace(/\@prefix\@/, buildPrefix));
+		} else if (s.search(/\@WITH_THREADS\@/) != -1) {
+			of.WriteLine(s.replace(/\@WITH_THREADS\@/, withThreads == "no"? "0" : "1"));
+		} else
+			of.WriteLine(ln);
+	}
+	ofi.Close();
+	of.Close();
 }
 
 /* Creates the readme file for the binary distribution of 'bname', for the
@@ -297,7 +338,7 @@ function genReadme(bname, ver, file)
 	f.WriteLine("a cheer of joy, a comment or a suggestion, feel free to contact me using");
 	f.WriteLine("the address below.");
 	f.WriteBlankLines(1);
-	f.WriteLine("                              Igor Zlatkovic (igor@stud.fh-frankfurt.de)");
+	f.WriteLine("                              Igor Zlatkovic (igor@zlatkovic.com)");
 	f.Close();
 }
 
@@ -317,7 +358,7 @@ for (i = 0; (i < WScript.Arguments.length) && (error == 0); i++) {
 		if (opt == "trio")
 			withTrio = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "threads")
-			withThreads = strToBool(arg.substring(opt.length + 1, arg.length));
+			withThreads = arg.substring(opt.length + 1, arg.length);
 		else if (opt == "ftp")
 			withFtp = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "http")
@@ -348,6 +389,10 @@ for (i = 0; (i < WScript.Arguments.length) && (error == 0); i++) {
 			withSchemas = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "regexps")
 			withRegExps = strToBool(arg.substring(opt.length + 1, arg.length));
+		else if (opt == "python")
+			withPython = strToBool(arg.substring(opt.length + 1, arg.length));
+		else if (opt == "compiler")
+			compiler = arg.substring(opt.length + 1, arg.length);
 		else if (opt == "debug")
 			buildDebug = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "static")
@@ -404,17 +449,29 @@ if (error != 0) {
 	WScript.Echo("Configuration failed, aborting.");
 	WScript.Quit(error);
 }
+if (withPython == true) {
+	configureLibxmlPy();
+	if (error != 0) {
+		WScript.Echo("Configuration failed, aborting.");
+		WScript.Quit(error);
+	}
+}
 
 // Create the makefile.
 var fso = new ActiveXObject("Scripting.FileSystemObject");
-fso.CopyFile(".\\Makefile.msvc", ".\\Makefile", true);
+var makefile = ".\\Makefile.msvc";
+if (compiler == "mingw")
+	makefile = ".\\Makefile.mingw";
+else if (compiler == "bcb")
+	makefile = ".\\Makefile.bcb";
+fso.CopyFile(makefile, ".\\Makefile", true);
 WScript.Echo("Created Makefile.");
 
 // Display the final configuration. 
 var txtOut = "\nXML processor configuration\n";
 txtOut += "---------------------------\n";
 txtOut += "              Trio: " + boolToStr(withTrio) + "\n";
-txtOut += "     Thread safety: " + boolToStr(withThreads) + "\n";
+txtOut += "     Thread safety: " + withThreads + "\n";
 txtOut += "        FTP client: " + boolToStr(withFtp) + "\n";
 txtOut += "       HTTP client: " + boolToStr(withHttp) + "\n";
 txtOut += "    HTML processor: " + boolToStr(withHtml) + "\n";
@@ -430,9 +487,11 @@ txtOut += "  Debugging module: " + boolToStr(withDebug) + "\n";
 txtOut += "  Memory debugging: " + boolToStr(withMemDebug) + "\n";
 txtOut += "    Regexp support: " + boolToStr(withRegExps) + "\n";
 txtOut += "XML Schema support: " + boolToStr(withSchemas) + "\n";
+txtOut += "   Python bindings: " + boolToStr(withPython) + "\n";
 txtOut += "\n";
 txtOut += "Win32 build configuration\n";
 txtOut += "-------------------------\n";
+txtOut += "          Compiler: " + compiler + "\n";
 txtOut += "     Debug symbols: " + boolToStr(buildDebug) + "\n";
 txtOut += "    Static xmllint: " + boolToStr(buildStatic) + "\n";
 txtOut += "    Install prefix: " + buildPrefix + "\n";
